import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;

/**
 * Enemy class which will be an enemy for the hero.
 * 
 * @author Chandrasekhar Thotakura
 * @version 1.0.0.
 */
public class Enemy extends MovingActor
{
    private Rings pow = new Rings();
    private static final int HEALTH_UNIT = 20;
    private static final int MAX_HEALTH = HEALTH_UNIT*2;
    private static final int ATTACK_POINTS = 50;
    private int direction;
    private int health;
    private State state;

    /**
     * Construtor
     */
    public Enemy()
    {
        direction = DIR_R;
        health = MAX_HEALTH;
        state = State.ACTIVE;
    }
        
    @Override
    public void act()
    {
        if(world.gameState == State.GAMERUNNING)
        {
            myAct();
        }
    }

    /**
     * myAct is same as act but executed only when the game is in running state
     */
    public void myAct()
    {
        if(state == State.ACTIVE)
        {
            autoMove();
            checkCollisionWithHero();
        }
        else if(state == State.GONE)
        {
            world.info.setInfo("   Well done !!!");
            world.addObject(new Blast(), getX(), getY());
            giveGift();
            world.removeObject(pow);
            world.removeObject(this);
        }
    }
    
    /**
     * giveGift will give out gift before being destroyed.
     */
    public void giveGift()
    {
        int random = Greenfoot.getRandomNumber(2);
        if(random == 0)
        {
            world.addObject(new Gift(), getX(), getY()); //coins
        }
    }
        
    /**
     * attackEnemy will make the enemy health decrease due to the hero's attack
     */
    public void attackEnemy() {
        if(state == State.ACTIVE)
        {
            health -= HEALTH_UNIT;
            if(health <= 0)
            {
                state = State.GONE;
                world.hero.gainEnemyKilledPoints(ATTACK_POINTS, getX(), getY());
            }
            else
            {
                world.hero.gainPoints(ATTACK_POINTS);
            }
        }
        else if(state == State.GONE)
        {
        }
    }
        
    /**
     * checkCollisionWithHero will attack the hero if the hero is in contact with enemy
     */
    private void checkCollisionWithHero()
    {
        if(getOneIntersectingObject(Hero.class) != null)
        {
            world.hero.attackHero();
        }
    }

    /**
     * Moves in the random direction
     */
    private void autoMove()
    {
        boolean flag = move(direction, 3.0);
        if(!flag)
        {
            if(direction == DIR_R)
            {
                direction = DIR_L;
            }
            else if(direction == DIR_L)
            {
                direction = DIR_R;
            }
            else if(direction == DIR_D)
            {
                direction = DIR_U;
            }
            else if(direction == DIR_U)
            {
                direction = DIR_D;
            }
        }
        pow.setLocation(getX(), getY());
        
        int randomBehaviour = Greenfoot.getRandomNumber(2);
        if(randomBehaviour == 1)
        {
            int randomDirection = Greenfoot.getRandomNumber(2);
            if(direction == DIR_R || direction == DIR_L)
            {
                randomDirection += 2;
            }
            if(isMovable(randomDirection, WALL_W+2))
            {
                direction = randomDirection;
            }
        }
    }

    /**
     * AddedToWorld
     */
    @Override
    public void addedToWorld(World w)
    {
        super.addedToWorld(w);
        w.addObject(pow, getX(), getY());
    }
}
