import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * This is the Game World class where the game logic runs.
 * 
 * @author Chandrasekhar Thotakura
 * @version 1.0.0
 */
public class GameWorld  extends World implements R
{
    protected Info info;
    protected Info levelInfo;
    protected Info scoreInfo;
    protected Info medalInfo;
    protected Info healthInfo;
    protected Info weaponsInfo;
    protected Info lines[];
    protected LevelTimer timer;
    protected Hero hero;
    protected GreenfootSound[] bgm;
    protected State gameState;
    protected int currentLevel;
    protected boolean isWaiting;

    /**
     * Constructor for objects of class GameWorld.
     * 
     */
    public GameWorld()
    {    
        super(CANVAS_W + DISPLAY_W, CANVAS_H, 1);
        prepare();
    }

    /**
     * Prepare the world for the start of the program. That is: create the initial
     * objects and add them to the world.
     */
    private void prepare()
    {
        //defines the paint order for the game
        setPaintOrder(Pointer.class, Hero.class, Enemy.class, Door.class, Power.class, Spawn.class, Wall.class);
        //defines the act order for the game
        setActOrder(GameWorld.class, CustomActor.class);

        bgm = new GreenfootSound[5];
        for(int i=0; i<bgm.length - 1; i++)
        {
            bgm[i] = new GreenfootSound("bgm" + (i + 1) + ".mp3");
        }
        bgm[4] = new GreenfootSound("bgm4.mp3");
        
        currentLevel = 0;
        hero = new Hero();
        isWaiting = false;

        gameState = State.GAME_SPLASH;
    }

    /**
     * act method for the World
     */
    @Override
    public void act()
    {
        if(gameState == State.GAME_SPLASH)
        {
            if(!isWaiting)
            {
                isWaiting = true;
                this.removeObjects(this.getObjects(CustomActor.class));
                setBackground("Splash.jpg");
                info = new Info("   Press Enter...", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                addObject(info, (CANVAS_W + DISPLAY_W)/2, CANVAS_H - 100);
            }
            else
            {
                updateKeyEvents(); // updates the key strokes from the user
            }            
        }
        else if(gameState == State.GAME_SPLASH_CONTROLS)
        {
            if(!isWaiting)
            {
                isWaiting = true;
                this.removeObjects(this.getObjects(CustomActor.class));
                setBackground("ControlScreen.jpg");
                info = new Info("   Press Enter...", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                addObject(info, (CANVAS_W + DISPLAY_W)/2, CANVAS_H - 100);
            }
            else
            {
                updateKeyEvents(); // updates the key strokes from the user
            }            
        }
        else if(gameState == State.GAME_RECURSION_SCREEN)
        {
            if(!isWaiting)
            {
                isWaiting = true;
                
                //IMPORTANT: first remove all objects
                this.removeObjects(this.getObjects(CustomActor.class));
            
                setBackground("black1.jpg");

                lines = new Info[13];
                lines[0] = new Info("                    01  escapeMaze(level i) {", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[1] = new Info("                    02     get the power", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[2] = new Info("                    03     kill all enemies", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[3] = new Info("                    04     if (found key) {", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[4] = new Info("                    05        return", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[5] = new Info("                    06     } else {", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[6] = new Info("                    07        escapeMaze(i+1)", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[7] = new Info("                    08     }", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[8] = new Info("                    09     collect coins", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[9] = new Info("                    10     return", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[10]= new Info("                    11  }", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                for (int i=0; i < lines.length - 2; i++) {
                    addObject(lines[i], (CANVAS_W + DISPLAY_W)/2, 200+i*20);
                }
                String message1 = "   This game explains the concept of RECURSION taking a recursive function as an example.";
                String message2 = "   The pseudocode for the recursive function in the example is as follows.";
                lines[11] = new Info(message1, true, java.awt.Color.WHITE, java.awt.Color.BLACK, CANVAS_W + DISPLAY_W);
                lines[12] = new Info(message2, true, java.awt.Color.WHITE, java.awt.Color.BLACK, CANVAS_W + DISPLAY_W);
                addObject(lines[11], (CANVAS_W + DISPLAY_W)/2, 120);
                addObject(lines[12], (CANVAS_W + DISPLAY_W)/2, 140);

                info = new Info("   Press Enter...", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                addObject(info, (CANVAS_W + DISPLAY_W)/2, CANVAS_H - 100);
            }
            else
            {
                updateKeyEvents(); // updates the key strokes from the user
            }            
        }
        else if(gameState == State.GAME_LEVEL_INTRO)
        {
            int recursionLevel = currentLevel + 1;
            if(currentLevel == 3)
            {
                recursionLevel = 2;
            }
            else if(currentLevel == 4)
            {
                recursionLevel = 1;
            }
            
            if(!isWaiting)
            {
                isWaiting = true;
                
                //IMPORTANT: first remove all objects
                this.removeObjects(this.getObjects(CustomActor.class));
            
                setBackground("black1.jpg");

                info = new Info("   Press Enter...", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                addObject(info, (CANVAS_W + DISPLAY_W)/2, CANVAS_H - 100);

                levelInfo = new Info("                    Level: ", true, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                if(recursionLevel >= currentLevel)
                {
                    levelInfo.setInfo("" + recursionLevel + " (forward or folding recursion)");
                }
                else
                {
                    levelInfo.setInfo("" + recursionLevel + " (backward or unfolding recursion)");
                }
                addObject(levelInfo, (CANVAS_W + DISPLAY_W)/2, CANVAS_H/2);
                /*
                addObject(levelInfo, (CANVAS_W + DISPLAY_W)/2, 120);
                lines = new Info[11];
                lines[0] = new Info("                    01  escapeMaze(level "+recursionLevel+") {", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[1] = new Info("                    02     get the power", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[2] = new Info("                    03     kill all enemies", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[3] = new Info("                    04     if (found key) {", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[4] = new Info("                    05        return", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[5] = new Info("                    06     } else {", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[6] = new Info("                    07        escapeMaze("+recursionLevel+" \+ 1)", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[7] = new Info("                    08     }", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[8] = new Info("                    09     collect coins", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[9] = new Info("                    10     return", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                lines[10]= new Info("                    11  }", false, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
                for (int i=0; i < lines.length; i++) {
                    addObject(lines[i], (CANVAS_W + DISPLAY_W)/2, 200+i*20);
                }
                */
            }
            else
            {
                updateKeyEvents(); // updates the key strokes from the user
            }            
        }
        else if(gameState == State.GAME_LOAD_LEVEL)
        {
            setBackground("granite-light.jpg");
            GreenfootImage img = getBackground();
            img.setColor(java.awt.Color.GRAY);
            img.fillRect(0, CANVAS_H - 2*WALL_W, CANVAS_W, 2*WALL_W); // horizontal
            img.setColor(java.awt.Color.BLACK);
            img.fillRect(CANVAS_W, 0, DISPLAY_W, CANVAS_H);//vertical
            setBackground(img);

            //IMPORTANT: first remove all objects
            this.removeObjects(this.getObjects(CustomActor.class));
            //this.removeObject(levelInfo);
            
            initLevel(currentLevel);
            timer.start();

            //gameState = State.GAMERUNNING;
            if(currentLevel <= 2)
            {
                changeCodeLine(2, 2);
            }
            else
            {
                changeCodeLine(9, 9);
            }
        }
        else if(gameState == State.WAIT_FOR_USER_CLICK)
        {
            if(!isWaiting)
            {
                isWaiting = true;
            }
            else
            {
                info.setInfo("   Click on the highlight line to continue.");
            }
        }
        else if(gameState == State.GAMERUNNING)
        {
            if(!bgm[currentLevel].isPlaying())
            {
                bgm[currentLevel].playLoop();
            }
            
            updateKeyEvents(); // updates the key strokes from the user
        }
        else if(gameState == State.GAME_COMPLETE)
        {
            this.removeObjects(this.getObjects(CustomActor.class));
            setBackground("black1.jpg");

            info = new Info("   Congratulations!!!", true, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
            addObject(info, (CANVAS_W + DISPLAY_W)/2, CANVAS_H/2-30);

            scoreInfo = new Info("   Your Score: ", true, java.awt.Color.BLUE, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
            hero.gainPoints(0);
            addObject(scoreInfo, (CANVAS_W + DISPLAY_W)/2, CANVAS_H/2);

            medalInfo = new Info("   Your Medals: ", true, java.awt.Color.MAGENTA, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
            hero.gainMedals(0);
            addObject(medalInfo, (CANVAS_W + DISPLAY_W)/2, CANVAS_H/2+30);

            Greenfoot.stop();
        }
        else if(gameState == State.GAMEOVER)
        {
            this.removeObjects(this.getObjects(CustomActor.class));
            setBackground("black1.jpg");

            info = new Info("   You Lost! Try Again!!", true, java.awt.Color.RED, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
            addObject(info, (CANVAS_W + DISPLAY_W)/2, CANVAS_H/2-30);

            scoreInfo = new Info("   Your Score: ", true, java.awt.Color.BLUE, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
            hero.gainPoints(0);
            addObject(scoreInfo, (CANVAS_W + DISPLAY_W)/2, CANVAS_H/2);

            medalInfo = new Info("   Your Medals: ", true, java.awt.Color.MAGENTA, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W);
            hero.gainMedals(0);
            addObject(medalInfo, (CANVAS_W + DISPLAY_W)/2, CANVAS_H/2+30);

            Greenfoot.stop();
        }
    }

    /**
     * changeCodeLine will change the line to the desired line number in the code.
     */
    public void changeCodeLine(int currentLine, int newLine)
    {
        lines[currentLine - 1].changeBackground(java.awt.Color.BLACK);
        lines[newLine - 1].changeBackground(java.awt.Color.RED);
        gameState = State.WAIT_FOR_USER_CLICK;
        addObject(new Pointer(), CANVAS_W - 40, 340+(newLine - 1)*20);
    }
    
    /**
     * resumeAfterMouseClick will continue the game.
     */
    public void resumeAfterMouseClick()
    {
        removeObjects(getObjects(Pointer.class));
        isWaiting = false;
        gameState = State.GAMERUNNING;
    }
    
    /**
     * initLevel will initiate the levels in the game.
     */
    private void initLevel(int cLevel)
    {
        //info = new Info();
        info = new Info("", false, java.awt.Color.BLACK, java.awt.Color.WHITE, CANVAS_W + DISPLAY_W, 2*WALL_W, 20);
        levelInfo = new Info("   Level: ", false, java.awt.Color.ORANGE, java.awt.Color.WHITE, DISPLAY_W);
        medalInfo = new Info("   Medals: ", false, java.awt.Color.GREEN, java.awt.Color.WHITE, DISPLAY_W);
        scoreInfo = new Info("   Score: ", false, java.awt.Color.GREEN, java.awt.Color.WHITE, DISPLAY_W);
        healthInfo = new Info("   Health: ", false, java.awt.Color.RED, java.awt.Color.WHITE, DISPLAY_W);
        weaponsInfo = new Info("   Weapons: ", false, java.awt.Color.RED, java.awt.Color.WHITE, DISPLAY_W);
        int timerValue = (150 - 10*cLevel);
        timer = new LevelTimer(timerValue,"   Timer: ", false, java.awt.Color.ORANGE, java.awt.Color.WHITE, DISPLAY_W);
        hero.prepareForNextLevel();

        int recursionLevel = cLevel + 1;
        if(cLevel == 3)
        {
            recursionLevel = 2;
        }
        else if(cLevel == 4)
        {
            recursionLevel = 1;
        }

        lines = new Info[11];
        if(recursionLevel >= cLevel)
        {
            lines[0] = new Info("01 escapeMaze(level "+recursionLevel+") {", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[1] = new Info("02    get the power", true, java.awt.Color.RED, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[2] = new Info("03    kill all enemies", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[3] = new Info("04    if (found key) {", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[4] = new Info("05       return", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[5] = new Info("06    } else {", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[6] = new Info("07       escapeMaze("+recursionLevel+"+1)", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[7] = new Info("08    }", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[8] = new Info("09    collect coins", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[9] = new Info("10    return", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[10]= new Info("11 }", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
        }
        else
        {
            lines[0] = new Info("01 escapeMaze(level "+recursionLevel+") {", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[1] = new Info("02    get the power", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[2] = new Info("03    kill all enemies", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[3] = new Info("04    if (found key) {", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[4] = new Info("05       return", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[5] = new Info("06    } else {", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[6] = new Info("07       escapeMaze("+recursionLevel+"+1)", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[7] = new Info("08    }", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[8] = new Info("09    collect coins", true, java.awt.Color.RED, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[9] = new Info("10    return", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
            lines[10]= new Info("11 }", true, java.awt.Color.BLACK, java.awt.Color.WHITE, DISPLAY_W, 14);
        }

        //add maze actors
        buildMaze(cLevel);

        //add other actors
        info.setInfo("   You are in Level " + recursionLevel);
        //addObject(info, info.getWidth()/2, CANVAS_H - 3*info.getHeight()/2);
        addObject(info, info.getWidth()/2, CANVAS_H - info.getHeight()/2);

        levelInfo.setInfo("" + recursionLevel);
        addObject(levelInfo, CANVAS_W + levelInfo.getWidth()/2, 40);

        if(recursionLevel >= cLevel)
        {
            timer.setInfoPassive("" + timerValue);
            addObject(timer, CANVAS_W + timer.getWidth()/2, 70);
        }
        

        hero.gainPoints(0);
        addObject(scoreInfo, CANVAS_W + scoreInfo.getWidth()/2, 120);

        hero.gainMedals(0);
        addObject(medalInfo, CANVAS_W + scoreInfo.getWidth()/2, 150);

        hero.looseHealth(0);
        addObject(healthInfo, CANVAS_W + healthInfo.getWidth()/2, 200);

        weaponsInfo.setInfo(""+hero.WEAPON_LIMIT);
        addObject(weaponsInfo, CANVAS_W + weaponsInfo.getWidth()/2, 230);        

        Info headingInfo = new Info("RECURSIVE FUNCTION", false, java.awt.Color.ORANGE, java.awt.Color.WHITE, DISPLAY_W);
        addObject(headingInfo, CANVAS_W + headingInfo.getWidth()/2, 310);

        for (int i=0; i < lines.length; i++) {
            addObject(lines[i], CANVAS_W + DISPLAY_W/2, 340+i*20);
        }
    }

    /**
     * Updates the game world with the mazeId.
     */
    private void buildMaze(int mazeId)
    {
        for(int i=0; i<CANVAS_H/WALL_W; i++)
        {
            for(int j=0; j<CANVAS_W/WALL_W; j++)
            {
                //System.out.println("i = "+i);
                //System.out.println("j = "+j);
                if(LEVEL[mazeId][i][j] == 1)
                {
                    addObject(new Wall(), j*WALL_W+WALL_W/2, i*WALL_W+WALL_W/2);
                }
                else if(LEVEL[mazeId][i][j] == 2)
                {
                    addObject(new Spawn(), j*WALL_W+WALL_W, i*WALL_W+WALL_W);
                    addObject(new Enemy(), j*WALL_W+WALL_W, i*WALL_W+WALL_W);
                }
                else if(LEVEL[mazeId][i][j] == 3)
                {
                    addObject(new Power(), j*WALL_W+WALL_W, i*WALL_W+WALL_W);
                }
                else if(LEVEL[mazeId][i][j] == 4)
                {
                    addObject(new Door(), j*WALL_W+WALL_W, i*WALL_W+WALL_W);
                }
                else if(LEVEL[mazeId][i][j] == 5)
                {
                    addObject(hero, j*WALL_W+WALL_W, i*WALL_W+WALL_W);
                }
                else if(LEVEL[mazeId][i][j] == 6)
                {
                    addObject(new Key(), j*WALL_W+WALL_W, i*WALL_W+WALL_W);
                }
                else if(LEVEL[mazeId][i][j] == 7)
                {
                    addObject(new Gift(Gift.TYPE_COIN, 1000), j*WALL_W+WALL_W, i*WALL_W+WALL_W);
                }
            }
        }
    }

    /**
     * Updates the key events in the game world.
     */
    private void updateKeyEvents()
    {
        //handle the key events
        if(gameState == State.GAMERUNNING)
        {
            if(Greenfoot.isKeyDown("d"))
            {
                hero.move(DIR_R);
            }
            else if(Greenfoot.isKeyDown("a"))
            {
                hero.move(DIR_L);
            }
            else if(Greenfoot.isKeyDown("s"))
            {
                hero.move(DIR_D);
            }
            else if(Greenfoot.isKeyDown("w"))
            {
                hero.move(DIR_U);
            }

            String key = Greenfoot.getKey();
            if(key != null)
            {
                if(key.equals(DIR[DIR_L]))
                {
                    hero.fire(DIR_L);
                }
                else if(key.equals(DIR[DIR_R]))
                {
                    hero.fire(DIR_R);
                }
                else if(key.equals(DIR[DIR_U]))
                {
                    hero.fire(DIR_U);
                }
                else if(key.equals(DIR[DIR_D]))
                {
                    hero.fire(DIR_D);
                }
            }
        }
        else if(gameState == State.GAME_SPLASH)
        {
            String key = Greenfoot.getKey();
            if(key != null)
            {
                if(key.equals("enter"))
                {
                    //change state
                    isWaiting = false;
                    gameState = State.GAME_SPLASH_CONTROLS;
                }
            }
        }
        else if(gameState == State.GAME_SPLASH_CONTROLS)
        {
            String key = Greenfoot.getKey();
            if(key != null)
            {
                if(key.equals("enter"))
                {
                    //change state
                    isWaiting = false;
                    gameState = State.GAME_RECURSION_SCREEN;
                }
            }
        }
        else if(gameState == State.GAME_RECURSION_SCREEN)
        {
            String key = Greenfoot.getKey();
            if(key != null)
            {
                if(key.equals("enter"))
                {
                    //change state
                    isWaiting = false;
                    gameState = State.GAME_LEVEL_INTRO;
                }
            }
        }
        else if(gameState == State.GAME_LEVEL_INTRO)
        {
            String key = Greenfoot.getKey();
            if(key != null)
            {
                if(key.equals("enter"))
                {
                    //change state
                    isWaiting = false;
                    gameState = State.GAME_LOAD_LEVEL;
                }
            }
        }
    }
    
    /**
     * changeLevel will do the necessary tasks and changes the level.
     */
    public void changeLevel()
    {
        bgm[currentLevel].stop();
        if(currentLevel < 4)
        {
            //increment to the next level
            currentLevel++;
            gameState = State.GAME_LEVEL_INTRO;
        }
        else
        {
            gameState = State.GAME_COMPLETE;
        }
    }

    /**
     * Started
     */
    @Override
    public void started()
    {
        //bgm[currentLevel].playLoop();
    }

    /**
     * Stopped
     */
    @Override
    public void stopped()
    {
        if(bgm[currentLevel].isPlaying())
        {
            bgm[currentLevel].pause();
        }
    }
}
