import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * All the gifts in the game are the objects of this class
 * 
 * @author Chandrasekhar Thotakura
 * @version 1.0.0
 */
public class Gift  extends CustomActor
{
    //private static final int ATTACK_POINTS = 1500;
    private State state;
    private long generalTimer;
    private int activeTime;
    static final int TYPE_COIN = 0;
    static final int TYPE_TIMER = 1;
    static final int TYPE_HEALTH = 2;
    static final int TYPE_MEDAL = 3;
    private int type;

    /**
     * Constructor of the Gift class
     */
    public Gift()
    {
        this(0, 500);
    }
    
    /**
     * Constructor of the Gift class with the type
     */
    public Gift(int _type)
    {
        this(_type, 500);
    }
    
    /**
     * Constructor of the Gift class with the type, active time
     */
    public Gift(int _type, int _activeTime)
    {
        state = State.ACTIVE;
        generalTimer = 0;
        activeTime = _activeTime;
        
        type = _type;
        if(type == TYPE_COIN)
        {
            setImage("coin40.png");
        }
        else if(type == TYPE_TIMER)
        {
            setImage("timer40.png");
        }
        else if(type == TYPE_HEALTH)
        {
            setImage("herz30.png");
        }
        else if(type == TYPE_MEDAL)
        {
            setImage("medal40.png");
        }
        else
        {
            type = TYPE_COIN;
            setImage("coin40.png");
        }
    }
    
    /**
     * checkCollisions will check wheather hero touched this gift or not and deal with it
     */
    private void checkCollisions()
    {
        if(getOneIntersectingObject(Hero.class) != null)
        {
            if(type == TYPE_COIN)
            {
                Greenfoot.playSound("gain.mp3");
                world.hero.gainPoints(150);
            }
            else if(type == TYPE_TIMER)
            {
                Greenfoot.playSound("gain.mp3");
                world.timer.addTime(20);
                world.hero.gainPoints(500);
            }
            else if(type == TYPE_HEALTH)
            {
                Greenfoot.playSound("gain.mp3");
                world.hero.gainHealth(10);
                world.hero.gainPoints(500);
            }
            else if(type == TYPE_MEDAL)
            {
                Greenfoot.playSound("gain.mp3");
                world.hero.gainMedals(1);
                world.hero.gainPoints(1500);
            }
            world.info.setInfo("   Great :)");
            world.removeObject(this);
        }
    }
    
    @Override
    public void act() 
    {
        if(world.gameState == State.GAMERUNNING)
        {
            myAct();
        }
    }
    
    public void myAct() 
    {
        if(state == State.ACTIVE)
        {
            generalTimer++;
            if(generalTimer >= activeTime)
            {
                state = State.GONE;
            }
            else
            {
                checkCollisions();
            }
        }
        else if(state == State.GONE)
        {
            world.info.setInfo("   You missed your gift :(");
            world.removeObject(this);
        }
    }
}
