import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Main actor in the game also called as Hero
 * 
 * @author Chandrasekhar Thotakura
 * @version 1.0.0
 */
public class Hero  extends MovingActor
{
    private static final int HEALTH_UNIT = 5;
    private static final int MAX_HEALTH = HEALTH_UNIT*20;
    private static final int ATTACKED_POINTS = 10;
    private static final long HEAL_TIME = 100;
    public static final int WEAPON_LIMIT = 3;
    private int health;
    private int points;
    private int medals;
    private int weaponCount;
    private long hurtTime;
    private long enemyKilledLastTime;
    private long generalTimer;
    private boolean hasPower;
    private boolean hasCheckedOnce;
    private boolean isInIfStmt;
    private boolean blinkFlag;
    private State state;
    
    /**
     * Constructor for the hero
     */
    public Hero()
    {
        points = 0;  
        medals = 0;
        prepareForNextLevel();
    }
    
    /**
     * prepareForNextLevel will reset the hero for the next level
     */
    public void prepareForNextLevel()
    {
        setImage("hero40.png");
        state = State.ACTIVE;
        health = MAX_HEALTH;
        weaponCount = WEAPON_LIMIT;
        hurtTime = 0;
        enemyKilledLastTime = 0;
        generalTimer = 0;
        blinkFlag = false;
        hasPower = false;
        hasCheckedOnce = false;
        isInIfStmt = false;
    }
    
    @Override
    public void act()
    {
        if(world.gameState == State.GAMERUNNING)
        {
            myAct();
        }
    }

    /**
     * myAct is same as act but executed only when the game is in running state
     */
    public void myAct() 
    {
        if(hasCheckedOnce && isInIfStmt)
        {
            isInIfStmt = false;
            if(world.currentLevel < 2)
            {
                world.changeCodeLine(4, 7);
            }
            else if(world.currentLevel == 2)
            {
                world.changeCodeLine(4, 5);
            }
        }
        if(!hasCheckedOnce)
        {
            if(world.currentLevel < 2)
            {
                if(world.getObjects(Enemy.class).size() <= 0 && world.getObjects(Spawn.class).size() <= 0)
                {
                    hasCheckedOnce = true;
                    isInIfStmt = true;
                    //world.info.setInfo("   You can go to next level from the door.");
                    world.changeCodeLine(3, 4);
                }
            }
            else if(world.currentLevel == 2)
            {
                if(world.getObjects(Enemy.class).size() <= 0 && world.getObjects(Spawn.class).size() <= 0)
                {
                    hasCheckedOnce = true;
                    isInIfStmt = true;
                    //world.info.setInfo("   Great! Take the key and return back.");
                    world.changeCodeLine(3, 4);
                }
            }
            else if(world.currentLevel > 2)
            {
                if(world.getObjects(Gift.class).size() <= 0)
                {
                    hasCheckedOnce = true;
                    world.changeCodeLine(9, 10);
                }
            }
        }
        
        generalTimer++;
        
        if(state == State.ACTIVE)
        {
        }
        else if(state == State.HEALING)
        {
            //if(System.currentTimeMillis() - hurtTime >= HEAL_TIME)
            if(generalTimer - hurtTime >= HEAL_TIME)
            {
                this.getImage().setTransparency(255);
                state = State.ACTIVE;
                return;
            }
            else
            {
                int trans = this.getImage().getTransparency();
                //System.out.println(trans);
                if(blinkFlag)
                {
                    trans -= 15;
                    if(trans < 100)
                    {
                        blinkFlag = false;
                        return;
                    }
                    this.getImage().setTransparency(trans);
                }
                else
                {
                    trans += 15;
                    if(trans > 255)
                    {
                        blinkFlag = true;
                        return;
                    }
                    this.getImage().setTransparency(trans);
                }
            }
        }
        else if(state == State.GONE)
        {
            world.info.setInfo("   Game Over");
            world.gameState = State.GAMEOVER;
        }
    }    

    /**
     * attackHero will make the hero's health decrease by some amount
     */
    public void attackHero() {
        if(state == State.ACTIVE)
        {
            //hurtTime = System.currentTimeMillis();
            hurtTime = generalTimer;
            looseHealth(HEALTH_UNIT);
            Greenfoot.playSound("herohurt.mp3");
            gainPoints(-ATTACKED_POINTS);
            this.getImage().setTransparency(100);
            world.info.setInfo("   Ouch !!!");
            if(health > 0)
            {
                state = State.HEALING;
            }
            else
            {
                state = State.GONE;
            }
        }
        else if(state == State.HEALING)
        {
        }
        else if(state == State.GONE)
        {
        }
    }
    
    /**
     * gainHealth will increase the health of the hero
     */
    public void gainHealth(int amount)
    {
        health += amount;
        
        //Dr.Karen requirement
        //world.healthInfo.setInfo(""+health+"%");
        world.healthInfo.setInfoPassive(""+health+"%");
    }
    
    /**
     * looseHealth will decrease the health of the hero
     */
    public void looseHealth(int amount)
    {
        health -= amount;
        
        //Dr.Karen requirement
        if(health % 10 == 0)
        {
            world.healthInfo.setInfo(""+health+"%");
        }
        else
        {
            world.healthInfo.setInfoPassive(""+health+"%");
        }
    }
    
    /**
     * gainEnemyKilledPoints will increase the points of the hero for his  great performance
     */
    public void gainEnemyKilledPoints(int amount, int enemyX, int enemyY)
    {
        if(generalTimer - enemyKilledLastTime < 200)
        {
            amount += 50;
            world.addObject(new Gift(Gift.TYPE_MEDAL), enemyX, enemyY); //MEDAL
        }
        else if(generalTimer - enemyKilledLastTime < 600)
        {
            amount += 25;
        }
        else if(generalTimer - enemyKilledLastTime < 1000)
        {
            amount += 15;
        }
        
        enemyKilledLastTime = generalTimer;
        
        gainPoints(amount);
    }
    
    /**
     * gainPoints will increase the points of the hero
     */
    public void gainPoints(int amount)
    {
        points += amount;
        
        //Dr.Karen requirement
        //world.scoreInfo.setInfo(""+points);
        world.scoreInfo.setInfoPassive(""+points);
    }
    
    /**
     * gainPower will give the hero the power to shoot
     */
    public void gainPower()
    {
        hasPower = true;
        setImage("heroPow40.png");
    }
    
    /**
     * gainMedal will increase the medals gained by hero
     */
    public void gainMedals(int amount)
    {
        medals += amount;
        
        //Dr.Karen requirement
        //world.medalInfo.setInfo(""+medals);
        world.medalInfo.setInfoPassive(""+medals);
    }
    
    /**
     * fire will shoot the bullent in the given direction
     */
    public void fire(int direction)
    {
        if(!hasPower)
        {
            if(world.currentLevel <= 2)
            {
                world.info.setInfo("   You should take POW to be able to shoot.");
            }
            return;
        }
        
        if(weaponCount > 0)
        {
            weaponCount--;
            Greenfoot.playSound("firegun.mp3");
            world.addObject(new Weapon(direction), getX(), getY());
        }
        else
        {
            Greenfoot.playSound("emptygun.mp3");
        }

        if(weaponCount > 0)
        {
            world.weaponsInfo.setInfoPassive(""+weaponCount);
        }
        else
        {        
            //Dr.Karen requirement
            //world.weaponsInfo.setInfo("0");
            world.weaponsInfo.setInfoPassive("0");
        }
    }
    
    /**
     * increases the Weapon Count
     */
    public void increaseWeaponCount()
    {
        this.weaponCount++;
        world.weaponsInfo.setInfoPassive(""+weaponCount);
    }
    
    /**
     * move
     */
    @Override
    public boolean move(int direction) {
        if(state != State.GONE)
        {
            return super.move(direction);
        }
        return false;
    }
    
    /**
     * move
     */
    @Override
    public boolean move(int direction, double speed) {
        if(state != State.GONE)
        {
            return super.move(direction, speed);
        }
        return false;
    }    
}
