import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * The blinking mechanism used in the game
 * 
 * @author Chandrasekhar Thotakura
 * @version 1.0.0
 */
public class Info  extends CustomActor
{
    private String currentText = "SR";
    private String padding;
    private boolean isStatic;
    private long disappearTimer;
    private int transparency;
    private java.awt.Color bg;
    private java.awt.Color fg;
    private static int TRANSPARENCY_LIMIT = 255;
    
    /**
     * Constructor
     */
    public Info()
    {
        this("", false, java.awt.Color.BLACK, java.awt.Color.WHITE, CANVAS_W);
    }

    /**
     * Constructor
     */
    public Info(String _padding)
    {
        this(_padding, false, java.awt.Color.BLACK, java.awt.Color.WHITE, CANVAS_W);
    }

    /**
     * Constructor
     */
    public Info(String _padding, boolean _isStatic, java.awt.Color _bg, java.awt.Color _fg, int _width)
    {
        this(_padding, _isStatic, _bg, _fg, _width, WALL_W, 18);
    }

    /**
     * Constructor
     */
    public Info(String _padding, boolean _isStatic, java.awt.Color _bg, java.awt.Color _fg, int _width, int _fontSize)
    {
        this(_padding, _isStatic, _bg, _fg, _width, WALL_W, _fontSize);
    }

    /**
     * Constructor
     */
    public Info(String _padding, boolean _isStatic, java.awt.Color _bg, java.awt.Color _fg, int _width, int _height, int _fontSize)
    {
        padding = _padding;
        isStatic = _isStatic;
        bg = _bg;
        fg = _fg;
        GreenfootImage img = getImage();
        img.setFont(new java.awt.Font("Courier New", java.awt.Font.PLAIN , _fontSize));
        img.scale(_width, _height);
        setImage(img);
        setInfo("");
    }

    /**
     * setInfo will set the info with blinging effect
     */
    public void setInfo(String s)
    {
        currentText = s;
        GreenfootImage img = getImage();
        img.clear();
        img.setColor(bg);
        img.fill();
        img.setColor(fg);
        if(getHeight() > WALL_W)
        {
            img.drawString(padding+currentText, 0, 25);
        }
        else
        {
            img.drawString(padding+currentText, 0, 15);
        }
        setImage(img);
        if(!isStatic)
        {
            disappearTimer = System.currentTimeMillis();
            transparency = TRANSPARENCY_LIMIT;
        }
    }
    
    /**
     * setInfoPassive will set the info without blinging effect
     */
    public void setInfoPassive(String s)
    {
        if(padding.length() > 0)
        {
            currentText = s;
            GreenfootImage img = getImage();
            img.clear();
            img.setColor(fg);
            img.drawString(padding+currentText, 0, 15);
            setImage(img);
        }
    }
    
    /**
     * changeBackground will change the blinking background for the info
     */
    public void changeBackground(java.awt.Color currentBg)
    {
        GreenfootImage img = getImage();
        img.clear();
        bg = currentBg;
        img.setColor(bg);
        img.fill();
        img.setColor(fg);
        img.drawString(padding+currentText, 0, 15);
        setImage(img);
    }
    
    @Override
    public void act() 
    {
        myAct();
        if (world.gameState == State.WAIT_FOR_USER_CLICK && Greenfoot.mouseClicked(this) && bg == java.awt.Color.RED)
        {
            world.resumeAfterMouseClick();
        }
    }
    
    public void myAct() 
    {
        long currentTimer = System.currentTimeMillis();
        if(!isStatic && currentTimer - disappearTimer > 0 && transparency > 1)
        {
            disappearTimer = currentTimer;
            transparency -= 2;
            
            if(padding.length() > 0)
            {
                changeBackground(new java.awt.Color(bg.getRed(), bg.getGreen(), bg.getBlue(), transparency));
            }
            else
            {
                GreenfootImage img = getImage();
                img.setTransparency(transparency);
                setImage(img);
            }
        }
    }
}
