import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Spawn class will generate the enemies in the game
 * 
 * @author Chandrasekhar Thotakura
 * @version 1.0.0
 */
public class Spawn  extends CustomActor
{
    private int healTimer;
    private int spawnTimer;
    //private int rotationTimer;
    //private static final long HEAL_TIME = 100;
    private static final long HEAL_TIME = 60;
    private final static int SPAWN_TIME = 1000;
    //private final static int ROTATE_SPEED = 10;
    private static final int HEALTH_UNIT = 5;
    //private static final int MAX_HEALTH = HEALTH_UNIT*10;
    private static final int MAX_HEALTH = HEALTH_UNIT*3;
    private static final int ATTACK_POINTS = 100;
    private int health;
    private State state;
    
    /**
     * Constructor
     */
    protected Spawn()
    {
        spawnTimer = 0;
        healTimer = 0;
        //rotationTimer = 0;
        health = MAX_HEALTH;
        state = State.ACTIVE;
    }
    
    @Override
    public void act() 
    {
        if(world.gameState == State.GAMERUNNING)
        {
            myAct();
        }
    }
    
    public void myAct()
    {
        //world.info.setInfo("   "+timer);
        if(state == State.ACTIVE)
        {
            if(spawnTimer >= SPAWN_TIME)
            {
                spawnTimer = 0;
                world.addObject(new Enemy(), getX(), getY());
            }
            else
            {
                spawnTimer++;
            }
            
            //rotate();
            /*if(rotationTimer >= ROTATE_SPEED)
            {
                rotationTimer = 0;
                rotate();
            }
            else
            {
                rotationTimer++;
            }*/
        }
        else if(state == State.HEALING)
        {
            //world.info.setInfo("   healing "+healTimer);
            if(healTimer >= HEAL_TIME)
            {
                state = State.ACTIVE;
                healTimer = 0;
                this.getImage().setTransparency(255);
            }
            else
            {
                healTimer++;
            }
        }
        else if(state == State.GONE)
        {
            world.info.setInfo("   Very Good !!!");
            world.addObject(new Blast(), getX(), getY());
            giveGift();
            world.removeObject(this);
        }
    }
    
    /**
     * giveGift will give out gift before being destroyed.
     */
    public void giveGift()
    {
        int random = Greenfoot.getRandomNumber(7);
        if(random == 0)
        {
            world.addObject(new Gift(Gift.TYPE_COIN), getX(), getY()); //coins
        }
        else if(random == 1)
        {
            world.addObject(new Gift(Gift.TYPE_TIMER), getX(), getY()); //timer
        }
        else if(random == 2)
        {
            world.addObject(new Gift(Gift.TYPE_HEALTH), getX(), getY()); //health
        }
    }
        
    /**
     * rotate will rotate the image
     */
    public void rotate() {
        this.setRotation(this.getRotation() + 20);
    }
        
    /**
     * attackSpawn will make the Spawn point health decrease due to the hero's attack
     */
    public boolean attackSpawn() {
        if(state == State.ACTIVE)
        {
            health -= HEALTH_UNIT;
            this.getImage().setTransparency(100);
            if(health <= 0)
            {
                state = State.GONE;
                world.hero.gainEnemyKilledPoints(ATTACK_POINTS, getX(), getY());
            }
            else
            {
                state = State.HEALING;
                world.hero.gainPoints(ATTACK_POINTS);
            }
            return true;
        }
        else if(state == State.HEALING)
        {
        }
        else if(state == State.GONE)
        {
        }
        return false;
    }        
}
