import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Weapon is the bullet that hero shoots
 * 
 * @author Chandrasekhar Thotakura
 * @version 1.0.0
 */
public class Weapon extends CustomActor
{
    private int direction;
    
    /**
     * Constructor
     */
    public Weapon(int dir)
    {
        direction = dir;
    }
    
    @Override
    public void act() 
    {
        if(world.gameState == State.GAMERUNNING)
        {
            myAct();
        }
    }
    
    public void myAct() 
    {
        move();
        checkCollision();
    }
    
    /**
     * checkCollision
     */
    public void checkCollision()
    {
        if(getX() <= 0 || getX() >= CANVAS_W - 1 || getY() <= 0 || getY() >= CANVAS_H - 1)
        {
            this.dispose();
            return;
        }

        Enemy e = (Enemy)getOneIntersectingObject(Enemy.class);
        if(e != null)
        {
            Greenfoot.playSound("enemyhurt.mp3");
            e.attackEnemy();
            this.dispose();
            return;
        }
        
        Spawn s = (Spawn)getOneIntersectingObject(Spawn.class);
        if(s != null)
        {
            if(s.attackSpawn())
            {
                Greenfoot.playSound("enemyhurt.mp3");
                this.dispose();
                return;
            }
        }
        
        Wall w = (Wall)getOneIntersectingObject(Wall.class);
        if(w != null)
        {
            this.dispose();
            return;
        }        
    }
    
    /**
     * disposes the bullet
     */
    public void dispose()
    {
        world.hero.increaseWeaponCount();
        world.removeObject(this);
    }
    
    /**
     * moves in the direction shot by the hero
     */
    public void move()
    {
        int speed = 5;
        if(direction == DIR_L)
        {
            setLocation(getX() - speed, getY());
        }
        else if(direction == DIR_R)
        {
            setLocation(getX() + speed, getY());
        }
        else if(direction == DIR_U)
        {
            setLocation(getX(), getY() - speed);
        }
        else if(direction == DIR_D)
        {
            setLocation(getX(), getY() + speed);
        }
    }
}
